// Ejemplo FreeImage 7
// Conversin de mapas de bits
// Abril 2023, Salvador Pozo, Con Clase

#include <iostream>
#include <iomanip>
#include <Freeimage.h>
#include <cstring>
#include <cstdlib>

const char *TipoImagen(FREE_IMAGE_TYPE tipo);

int main(int argc, char**argv)
{
    FIBITMAP* dib;
	char *lpszPathName;
    FREE_IMAGE_FORMAT fif = FIF_UNKNOWN;
    RGBQUAD *pal;

	if(argc < 2) {
        printf("Usar: FreeImage007 <fichero>\n");
        return 0;
	}
	lpszPathName = argv[1];

	fif = FreeImage_GetFileType(lpszPathName, 0);
	if(fif == FIF_UNKNOWN) {
		fif = FreeImage_GetFIFFromFilename(lpszPathName);
	}
	if((fif != FIF_UNKNOWN) && FreeImage_FIFSupportsReading(fif)) {
		dib = FreeImage_Load(fif, lpszPathName, 0);
		std::cout << "Fichero: " << lpszPathName << std::endl;

		if(FreeImage_HasPixels(dib)) {
            pal = FreeImage_GetPalette(dib);
            std::cout << TipoImagen(FreeImage_GetImageType(dib)) << std::endl;
            if(pal) std::cout << FreeImage_GetColorsUsed(dib) << " Colores" << std::endl;

            // Conversiones:
            FreeImage_Save(FIF_BMP, FreeImage_Dither(dib, FID_BAYER4x4), "bayer4x4.bmp", BMP_DEFAULT);
            FreeImage_Save(FIF_BMP, FreeImage_Dither(dib, FID_CLUSTER6x6), "cluster8x8.bmp", BMP_DEFAULT);
            FreeImage_Save(FIF_BMP, FreeImage_Threshold(dib, 128), "threshold128.bmp", BMP_DEFAULT);
            FreeImage_Save(FIF_BMP, FreeImage_ConvertTo4Bits(dib), "4bits.bmp", BMP_DEFAULT);
            FreeImage_Save(FIF_BMP, FreeImage_ConvertTo8Bits(dib), "8bits.bmp", BMP_DEFAULT);
            FreeImage_Save(FIF_BMP, FreeImage_ConvertToGreyscale(dib), "escalagrises.bmp", BMP_DEFAULT);
            FreeImage_Save(FIF_BMP, FreeImage_ConvertToGreyscale(dib), "escalagrises.bmp", BMP_DEFAULT);
            FreeImage_Save(FIF_BMP, FreeImage_ColorQuantize(dib, FIQ_WUQUANT), "quantize.bmp", BMP_DEFAULT);
            FreeImage_Save(FIF_BMP, FreeImage_ConvertTo16Bits555(dib), "16bits555.bmp", BMP_DEFAULT);
            FreeImage_Save(FIF_BMP, FreeImage_ConvertTo32Bits(dib), "32bits.bmp", BMP_DEFAULT);
		}
    }
	FreeImage_Unload(dib);

    return 0;
}

const char *TipoImagen(FREE_IMAGE_TYPE tipo) {
    switch(tipo) {
        case FIT_UNKNOWN: return "Formato desconocido"; break;
        case FIT_BITMAP:  return "Imagen estandar: 1-, 4-, 8-, 16-, 24-, 32-bit"; break;
        case FIT_UINT16:  return "Array de unsigned short"; break;
        case FIT_INT16:   return "Array de short"; break;
        case FIT_UINT32:  return "Array de unsigned long"; break;
        case FIT_INT32:   return "Array de long"; break;
        case FIT_FLOAT:   return "Array de float"; break;
        case FIT_DOUBLE:  return "Array de double"; break;
        case FIT_COMPLEX: return "Array de FICOMPLEX"; break;
        case FIT_RGB16:   return "Imagen RGB de 48-bit"; break;
        case FIT_RGBA16:  return "Imagen RGBA de 64-bit"; break;
        case FIT_RGBF:    return "Imagen RGB float de 96-bit"; break;
        case FIT_RGBAF:   return "Imagen RGBA float de 128-bit"; break;
    }
    return "?";
}
